"""Tests for elements helper functions"""


import numpy as np
from numpy.random import rand
import matplotlib.pyplot as plt

import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "../"))
import pyqlaw

def test_kep_sv():
    # initial and final elements: [a,e,i,RAAN,omega,ta]
    LU = 42164.0
    GM_EARTH = 398600.44
    VU = np.sqrt(GM_EARTH/LU)
    TU = LU/VU

    rp_gto = 200 + 6378
    ra_gto = 35786 + 6378
    sma_gto = (rp_gto + ra_gto)/(2*LU)
    ecc_gto = (ra_gto - rp_gto)/(ra_gto + rp_gto)
    KEP0 = np.array([sma_gto,ecc_gto,np.deg2rad(23),0.3,2.1,3.06])

    # convert to SV and back
    sv = pyqlaw.kep2sv(KEP0, GM_EARTH)
    kep_converted_back = pyqlaw.sv2kep(sv, GM_EARTH)

    # jit-compiled ver
    assert np.abs(pyqlaw.get_semiMajorAxis(sv, GM_EARTH) - KEP0[0]) <= 1e-12
    assert np.abs(np.linalg.norm(pyqlaw.get_eccentricity(sv, GM_EARTH)) - KEP0[1]) <= 1e-12
    assert np.abs(pyqlaw.get_inclination(sv)             - KEP0[2]) <= 1e-12
    assert np.abs(pyqlaw.get_raan(sv)                    - KEP0[3]) <= 1e-12
    assert np.abs(pyqlaw.get_omega(sv, GM_EARTH)         - KEP0[4]) <= 1e-12
    assert np.abs(pyqlaw.get_trueanom(sv, GM_EARTH)      - KEP0[5]) <= 1e-12
    assert np.abs(pyqlaw.get_period(sv, GM_EARTH) - 2*np.pi*np.sqrt(KEP0[0]**3/GM_EARTH)) <= 1e-12
    
    # pure python ver for coverage purposes
    assert np.abs(pyqlaw.get_semiMajorAxis.py_func(sv, GM_EARTH) - KEP0[0]) <= 1e-12
    assert np.abs(np.linalg.norm(pyqlaw.get_eccentricity.py_func(sv, GM_EARTH)) - KEP0[1]) <= 1e-12
    assert np.abs(pyqlaw.get_inclination.py_func(sv)             - KEP0[2]) <= 1e-12
    assert np.abs(pyqlaw.get_raan.py_func(sv)                    - KEP0[3]) <= 1e-12
    assert np.abs(pyqlaw.get_omega.py_func(sv, GM_EARTH)         - KEP0[4]) <= 1e-12
    assert np.abs(pyqlaw.get_trueanom.py_func(sv, GM_EARTH)      - KEP0[5]) <= 1e-12
    assert np.abs(pyqlaw.get_period.py_func(sv, GM_EARTH) - 2*np.pi*np.sqrt(KEP0[0]**3/GM_EARTH)) <= 1e-12
    
    # jit-compiled ver
    sv = pyqlaw.kep2sv(KEP0, GM_EARTH)
    kep_converted_back = pyqlaw.sv2kep(sv, GM_EARTH)
    assert all(np.abs(kep_converted_back - KEP0) <= 1e-12)
    
    # pure python ver for coverage purposes
    sv = pyqlaw.kep2sv.py_func(KEP0, GM_EARTH)
    kep_converted_back = pyqlaw.sv2kep.py_func(sv, GM_EARTH)
    assert all(np.abs(kep_converted_back - KEP0) <= 1e-12)


def test_kep_mee_a():
    # initial and final elements: [a,e,i,RAAN,omega,ta]
    LU = 42164.0
    GM_EARTH = 398600.44
    VU = np.sqrt(GM_EARTH/LU)
    TU = LU/VU

    rp_gto = 200 + 6378
    ra_gto = 35786 + 6378
    sma_gto = (rp_gto + ra_gto)/(2*LU)
    ecc_gto = (ra_gto - rp_gto)/(ra_gto + rp_gto)
    KEP0 = np.array([sma_gto,ecc_gto,np.deg2rad(23),0.3,2.1,3.06])
    kep_converted_back = pyqlaw.mee_with_a2kep(pyqlaw.kep2mee_with_a(KEP0))
    assert all(np.abs(kep_converted_back - KEP0) <= 1e-12)


if __name__ == "__main__":
    test_kep_sv()
    test_kep_mee_a()
    print("Done!")


"""
Test for constructing QLaw object
"""


import numpy as np
from numpy.random import rand
import matplotlib.pyplot as plt

import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "../"))
import pyqlaw


def test_solve_kep(close_figures=True):
    # construct problem
    prob = pyqlaw.QLaw(
        elements_type="keplerian",
        integrator="rk4"
    )#, verbosity=2)
    # initial and final elements: [a,e,i,RAAN,omega,ta]
    oe0 = np.array([1.5, 0.2, 0.3, 1e-2, 1e-2, 1e-2])
    oeT = np.array([2.2, 0.3, 1.1, 0.3, 0.0])
    woe = [1.0, 1.0, 1.0, 1.0, 1.0]
    # spacecraft parameters
    mass0 = 1.0
    tmax = 1e-3
    mdot = 1e-4
    tf_max = 3000.0
    t_step = 1.0
    # set problem
    prob.set_problem(oe0, oeT, mass0, tmax, mdot, tf_max, t_step, woe=woe)
    prob.pretty()

    # solve
    prob.solve()
    prob.pretty_results()

    # plot
    fig1, ax1 = prob.plot_elements_history(to_keplerian=False)
    fig2, ax2 = prob.plot_trajectory_3d(sphere_radius=0.1)
    fig3, ax3 = prob.plot_controls()
    fig4, ax4 = prob.plot_efficiency()
    fig5, ax5 = prob.plot_Q()
    fig6, ax6 = prob.plot_trajectory_2d()
    if close_figures:
        plt.close('all')
    assert prob.converge == True


if __name__=="__main__":
    test_solve_kep(close_figures=False)
    plt.show()
    print("Done!")


"""Test for eoms"""



import numpy as np
from numpy.random import rand
import matplotlib.pyplot as plt

import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), "../"))
import pyqlaw


def test_eom_kep():
    # construct problem
    prob = pyqlaw.QLaw(
        elements_type="keplerian",
        integrator="rk4"
    )#, verbosity=2)
    # initial and final elements: [a,e,i,RAAN,omega,ta]
    oe0 = np.array([1.5, 0.2, 0.3, 1e-2, 1e-2, 1e-2])
    oeT = np.array([2.2, 0.3, 1.1, 0.3, 0.0])
    woe = [1.0, 1.0, 1.0, 1.0, 1.0]
    # spacecraft parameters
    mass0 = 1.0
    tmax = 1e-3
    mdot = 1e-4
    tf_max = 3000.0
    t_step = 1.0
    # set problem
    prob.set_problem(oe0, oeT, mass0, tmax, mdot, tf_max, t_step, woe=woe)
    
    # query psi
    accel_thrust = tmax/mass0
    _, _, _, psi, _, _ = pyqlaw.lyapunov_control_angles(
        fun_lyapunov_control=prob.lyap_fun,
        mu=prob.mu, 
        f=accel_thrust, 
        oe=oe0, 
        oeT=prob.oeT, 
        rpmin=prob.rpmin, 
        m_petro=prob.m_petro, 
        n_petro=prob.n_petro, 
        r_petro=prob.r_petro, 
        b_petro=prob.b_petro, 
        k_petro=prob.k_petro, 
        wp=prob.wp, 
        woe=prob.woe,
    )
    u = np.array([0.1,0.4,0.2])
    u *= accel_thrust/np.linalg.norm(u)
    ode_params = (prob.mu, u, psi[0], psi[1], psi[2])

    # known check case
    doe_check = np.array([
        0.00392952589873677, 0.00209742303058534, 
        0.0004363521324628918, 2.9535057990649755e-05, 
        -0.001241443160063603, 0.8345326718595107])
    
    # evaluate eom (jit-ed)
    doe = pyqlaw.eom_kep_gauss(0.0, oe0, ode_params)
    assert all(np.abs(doe - doe_check)) <= 1e-12

    # evaluate eom (pure python)
    doe = pyqlaw.eom_kep_gauss.py_func(0.0, oe0, ode_params)
    assert all(np.abs(doe - doe_check)) <= 1e-12


def test_eom_mee_with_a():
    # construct problem
    prob = pyqlaw.QLaw(
        elements_type="mee_with_a",
        integrator="rk4"
    )#, verbosity=2)
    # initial and final elements: [a,e,i,RAAN,omega,ta]
    oe0 = pyqlaw.kep2mee_with_a(np.array([1.0, 1e-4, 0.0, 1e-2, 1e-2, 1e-3]))
    oeT = pyqlaw.kep2mee_with_a(np.array([1.8, 0.2, np.pi/6, 1e-2, 1e-2, 1e-3]))
    woe = [1.0, 1.0, 1.0, 1.0, 1.0]
    # spacecraft parameters
    mass0 = 1.0
    tmax = 1e-3
    mdot = 1e-4
    tf_max = 3000.0
    t_step = 1.0
    # set problem
    prob.set_problem(oe0, oeT, mass0, tmax, mdot, tf_max, t_step, woe=woe)
    
    # query psi
    accel_thrust = tmax/mass0
    _, _, _, psi, _, _ = pyqlaw.lyapunov_control_angles(
        fun_lyapunov_control=prob.lyap_fun,
        mu=prob.mu, 
        f=accel_thrust, 
        oe=oe0, 
        oeT=prob.oeT, 
        rpmin=prob.rpmin, 
        m_petro=prob.m_petro, 
        n_petro=prob.n_petro, 
        r_petro=prob.r_petro, 
        b_petro=prob.b_petro, 
        k_petro=prob.k_petro, 
        wp=prob.wp, 
        woe=prob.woe,
    )
    u = np.array([0.1,0.4,0.2])
    u *= accel_thrust/np.linalg.norm(u)
    ode_params = (prob.mu, u, psi[0], psi[1], psi[2])

    # known check case
    doe_check = np.array([
        0.0017459177484860002, 0.001749940431641845, -0.00018151195024652812,
        0.0002181479590836209, 4.581780682318711e-06, 1.0002000249029905])
    
    # evaluate eom (jit-ed)
    doe = pyqlaw.eom_mee_with_a_gauss(0.0, oe0, ode_params)
    #print(f"doe_check = {list(doe)}")
    assert all(np.abs(doe - doe_check)) <= 1e-12

    # evaluate eom (pure python)
    doe = pyqlaw.eom_mee_with_a_gauss.py_func(0.0, oe0, ode_params)
    assert all(np.abs(doe - doe_check)) <= 1e-12


if __name__=="__main__":
    test_eom_kep()
    test_eom_mee_with_a()
    print("Done!")